unit VirtualToOverride;

// Delphi 5 add in to change "virtual" to "override".
// When editing a subclass, copy methods from the base class
// into the derived class. Position the cursor in the virtual
// directive and press the magic keystroke (default is Ctrl+Alt+O)
// to change "virtual" or "dynamic" into "override". As an added
// bonus, if the method is abstract, the abstract directive
// is automatically deleted.
//
// Copyright  1999 Tempest Software, Inc.

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Dialogs, ToolsAPI;

type
  TVirtualToOverrideAddIn = class(TNotifierObject, IOTAWizard)
  private
    BindingIndex: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
  end;
  TShortCutArray = array of TShortCut;
  TVOKBinding = class(TNotifierObject, IOTAKeyboardBinding)
  private
    procedure Execute(const Context: IOTAKeyContext; Code: TShortCut; var Result: TKeyBindingResult);
    function GetShortCut: TShortCutArray;
  public
    function GetBindingType: TBindingType;
    function GetDisplayName: string;
    function GetName: string;
    procedure BindKeyboard(const Services: IOTAKeyBindingServices);
  end;

procedure Register;

implementation

uses Menus, Registry;

procedure Register;
begin
  RegisterPackageWizard(TVirtualToOverrideAddIn.Create);
end;

// Stub: Delphi never calls Execute for this add-in.
constructor TVirtualToOverrideAddIn.Create;
begin
  inherited;
  BindingIndex := (BorlandIDEServices as IOTAKeyboardServices).AddKeyboardBinding(TVOKBinding.Create);
end;

destructor TVirtualToOverrideAddIn.Destroy;
begin
  (BorlandIDEServices as IOTAKeyboardServices).RemoveKeyboardBinding(BindingIndex);
  inherited;
end;

procedure TVirtualToOverrideAddIn.Execute;
begin
end;

function TVirtualToOverrideAddIn.GetIDString: string;
begin
  Result := 'Tempest Software.VirtualToOverrideAddIn';
end;

function TVirtualToOverrideAddIn.GetName: string;
begin
  Result := 'Virtual to Override AddIn';
end;

function TVirtualToOverrideAddIn.GetState: TWizardState;
begin
  Result := [];
end;

{ TVOKBinding }

// Let the user specify a different keystroke sequence in the
// registry. Separate keystrokes with commas.
const
  RegKey = '\Software\Tempest Software';
  RegSection = 'Virtual to Override';
  RegEntry = 'Keys';
  DefaultKeys = 'Ctrl+Alt+O';

function TVOKBinding.GetShortCut: TShortCutArray;
var
  Reg: TRegIniFile;
  KeyText: string;
  Comma: Integer;
  Text: string;
begin
  SetLength(Result, 0);
  Reg := TRegIniFile.Create(RegKey);
  try
    KeyText := Reg.ReadString(RegSection, RegEntry, DefaultKeys);
  finally
    Reg.Free;
  end;

  repeat
    Comma := Pos(',', KeyText);
    if Comma <= 0 then
    begin
      Text := KeyText;
      KeyText := '';
    end
    else
    begin
      Text := Copy(KeyText, 1, Comma-1);
      Delete(KeyText, 1, Comma);
    end;
    SetLength(Result, Length(Result) + 1);
    Result[High(Result)] := TextToShortCut(Text);
  until KeyText = '';
end;

procedure TVOKBinding.BindKeyboard(const Services: IOTAKeyBindingServices);
var
  Keys: TShortCutArray;
begin
  Keys := GetShortCut;
  if Length(Keys) > 0 then
    Services.AddKeyBinding(Keys, Execute, nil, 0);
end;

procedure TVOKBinding.Execute(const Context: IOTAKeyContext;
  Code: TShortCut; var Result: TKeyBindingResult);
const
  IdentifierChars = ['a'..'z','A'..'Z','_'];
var
  Buffer: IOTAEditBuffer;
  Position: IOTAEditPosition;
  Identifier: string;
  I: Integer;
  OverrideText: string;
begin
  Buffer := (BorlandIDEServices as IOTAEditorServices).TopBuffer;
  if Buffer <> nil then
  begin
    Position := Buffer.EditPosition;
    Position.MoveCursor(mmSkipLeft or mmSkipWord);
    Identifier := Position.RipText(IdentifierChars, 0);
    if (CompareText(Identifier, 'virtual') = 0) or
       (CompareText(Identifier, 'dynamic') = 0) then
    begin
      // Delete always deletes one character, regardless of its
      // argument's value.
      for I := 1 to Length(Identifier) do
        Position.Delete(1);
      // Replace with "override", but try to match the capitalization
      // used in the original text.
      OverrideText := 'override';
      // Match an initial capital.
      if Identifier[1] in ['A'..'Z'] then
        OverrideText[1] := UpCase(OverrideText[1]);
      // If the last is also uppercase, assume the user
      // wants the entire word in uppercase.
      if Identifier[Length(Identifier)] in ['A'..'Z'] then
        for I := 2 to Length(OverrideText) do
          OverrideText[I] := UpCase(OverrideText[I]);
      // Insert the new directive.
      Position.InsertText(OverrideText);

      // Look for the abstract directive, and delete it if present.
      // Skip over the semicolon that follows override.
      Position.MoveCursor(mmSkipRight or mmSkipNonWord);
      // Grab the next identifier.
      Identifier := Position.RipText(IdentifierChars, 0);
      if CompareText(Identifier, 'abstract') = 0 then
      begin
        for I := 1 to Length(Identifier) do
          Position.Delete(1);
        // Delete the semicolon.
        Position.MoveCursor(mmSkipRight or mmSkipWhite);
        if Position.Read(1) = ';' then
          Position.Delete(1);
      end;
    end;
    // Make all changes visible.
    Buffer.TopView.Paint;
  end;
end;

function TVOKBinding.GetBindingType: TBindingType;
begin
  Result := btPartial;
end;

function TVOKBinding.GetDisplayName: string;
begin
  Result := 'Virtual to Override';
end;

function TVOKBinding.GetName: string;
begin
  Result := 'Tempest Software.Virtual To Override';
end;

end.
